@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -16,11 +16,15 @@
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 #include "application.h"
+#include "extension_set.h"
+#include "log.h"
 #include <mobius/database/connection_pool.h>
 #include <mobius/exception.inc>
-#include <mobius/exception_posix.inc>
-#include <mobius/uri.h>
+#include <mobius/io/path.h>
+#include <mobius/io/folder.h>
+#include <atomic>
 #include <cstdlib>
+#include <mutex>
 #include <stdexcept>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -32,30 +36,46 @@ namespace core
 namespace
 {
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief create directory recursively
-//! \param path directory path
+//! \brief Application data
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static void
-make_directory (const std::string& path)
-{
-  if (mkdir (path.c_str (), 0700) == -1)
-    {
-      if (errno == ENOENT)
-        {
-          auto idx = path.rfind ('/');
-          if (idx == std::string::npos)
-            throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("cannot create directory"));
+//! \brief application ID
+static const std::string id_ = "mobiusft";
 
-          std::string head = path.substr (0, idx);
-          std::string tail = path.substr (idx + 1);
-          make_directory (head);
-          make_directory (tail);
-        }
+//! \brief application name
+static const std::string name_ = "Mobius Forensic Toolkit";
 
-      else if (errno != EEXIST)
-        throw std::runtime_error (MOBIUS_EXCEPTION_POSIX);
-    }
-}
+//! \brief application version
+static const std::string version_ = "1.25";
+
+//! \brief application title
+static const std::string title_ = "Mobius v1.25";
+
+//! \brief application copyright
+static const std::string copyright_ = "Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar";
+
+//! \brief application OS name  
+static const std::string os_name_ = "x86_64-pc-linux-gnu";
+
+//! \brief data folder path
+static const std::string data_folder_ = "/home/aguiar/tmp/mobius.install/share/" + id_;
+
+//! \brief configuration folder path
+std::string config_folder_;
+
+//! \brief cache folder path
+std::string cache_folder_;
+
+//! \brief module's connection pool
+mobius::database::connection_pool connection_pool_;
+
+//! \brief C++ extensions
+static extension_set extensions_;
+
+//! \brief Is application data loaded
+std::atomic_bool is_loaded_ (false);
+
+//! \brief Is application running
+std::atomic_bool is_running_ (false);
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief join a relative path to a absolute path
@@ -63,73 +83,35 @@ make_directory (const std::string& path)
 //! \param rel_path relative path
 //! \return absolute path
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static const std::string
+static std::string
 join_path (const std::string& abs_path, const std::string& rel_path)
 {
-  mobius::uri url;
-  url.set_scheme ("file");
-  url.set_path (abs_path);
-  url.join_path (rel_path);
+  auto path = mobius::io::path (abs_path);
+  return to_string (path.get_child_by_path (rel_path));
+}
 
-  std::string fullpath = get_path_from_url (url.get_value ());
+} // namespace
 
-  if (fullpath.substr (0, abs_path.length ()) != abs_path)
-    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("invalid path"));
-
-  return fullpath;
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief get database object for current thread
+//! \return database object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::database::database
+get_database ()
+{
+  return connection_pool_.get_database ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief application data structure
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static struct
-{
-  //! \brief application ID
-  const std::string id = "mobiusft";
-
-  //! \brief application name
-  const std::string name = "Mobius Forensic Toolkit";
-
-  //! \brief application version
-  const std::string version = "1.12";
-
-  //! \brief application title
-  const std::string title = "Mobius v1.12";
-
-  //! \brief application copyright
-  const std::string copyright = "Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar";
-
-  //! \brief application OS name  
-  const std::string os_name = "x86_64-pc-linux-gnu";
-
-  //! \brief data folder path
-  const std::string data_folder = "/home/aguiar/tmp/mobius.install/share/" + id;
-  
-  //! \brief configuration folder path
-  std::string config_folder;
-
-  //! \brief cache folder path
-  std::string cache_folder;
-
-  //! \brief module's connection pool
-  mobius::database::connection_pool pool;
-  
-  //! \brief flag object loaded
-  bool is_loaded = false;
-
-} data_;
-
-// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief initialize data structure
+//! \brief Constructor
 //! \see https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-static void
-load_data ()
+application::application ()
 {
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // check if it is already loaded
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  if (data_.is_loaded)
+  if (is_loaded_)
     return;
 
   const char *home = getenv ("HOME");
@@ -138,16 +120,18 @@ load_data ()
   // config folder
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   const char *xdg_config_home = getenv ("XDG_CONFIG_HOME");
-      
+
   if (xdg_config_home)
-    data_.config_folder = xdg_config_home;
-
+    config_folder_ = xdg_config_home;
   else
-    data_.config_folder = home + std::string ("/.config");
+    config_folder_ = home + std::string ("/.config");
 
-  data_.config_folder += std::string ("/") + data_.id;
+  config_folder_ += std::string ("/") + id_;
 
-  make_directory (data_.config_folder);
+  auto config_folder = mobius::io::new_folder_by_path (config_folder_);
+  config_folder.create ();
+
+  set_logfile_path (config_folder_ + "/mobius.log");
       
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // cache folder
@@ -155,22 +139,23 @@ load_data ()
   const char *xdg_cache_home = getenv ("XDG_CACHE_HOME");
       
   if (xdg_cache_home)
-    data_.cache_folder = xdg_cache_home;
+    cache_folder_ = xdg_cache_home;
 
   else
-    data_.cache_folder = home + std::string ("/.cache");
+    cache_folder_ = home + std::string ("/.cache");
 
-  data_.cache_folder += std::string ("/") + data_.id;
+  cache_folder_ += std::string ("/") + id_;
 
-  make_directory (data_.cache_folder);
+  auto cache_folder = mobius::io::new_folder_by_path (cache_folder_);
+  cache_folder.create ();
       
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // database
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  const std::string db_path = join_path (data_.config_folder, data_.id + ".sqlite");
-  data_.pool.set_path (db_path);
+  const std::string db_path = join_path (config_folder_, id_ + ".sqlite");
+  connection_pool_.set_path (db_path);
 
-  auto db = data_.pool.get_database ();
+  auto db = connection_pool_.get_database ();
   auto transaction = db.new_transaction ();
 
   db.execute ("PRAGMA foreign_keys = ON;");
@@ -186,20 +171,66 @@ load_data ()
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // set data loaded
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  data_.is_loaded = true;
+  is_loaded_ = true;
 }
 
-} // namespace
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Start application
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+void
+application::start ()
+{
+  mobius::core::log log (__FILE__, __FUNCTION__);
+  log.info (__LINE__, "starting application");
+
+  // check if application is already running
+  if (is_running_)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("application is already running"));
+
+  // search installed C++ extensions
+  log.info (__LINE__, "loading C++ extensions");
+  auto extensions_path = get_config_path ("extensions");
+  auto folder = mobius::io::new_folder_by_path (extensions_path);
+  
+  for (auto& c : folder.get_children ())
+    {
+      if (c.is_file () && c.get_extension () == "so")
+        extensions_.load (c.get_path ());
+    }
+
+  // start extensions
+  for (auto& e : extensions_.get_extensions ())
+    e.start ();
+
+  // set application running
+  log.info (__LINE__, "application started");
+  is_running_ = true;
+}
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief get database object for current thread
-//! \return database object
+//! \brief Stop application
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-mobius::database::database
-get_database ()
+void
+application::stop ()
 {
-  load_data ();
-  return data_.pool.get_database ();
+  mobius::core::log log (__FILE__, __FUNCTION__);
+
+  // check if application is running
+  if (!is_running_)
+    throw std::runtime_error (MOBIUS_EXCEPTION_MSG ("application is not running"));
+
+  // stop extensions
+  log.info (__LINE__, "unloading C++ extensions");
+
+  for (auto& e : extensions_.get_extensions ())
+    e.stop ();
+
+  // remove extensions
+  extensions_.clear ();
+
+  // set running false  
+  log.info (__LINE__, "application stopped");
+  is_running_ = false;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -208,7 +239,7 @@ get_database ()
 std::string
 application::get_id () const
 {
-  return data_.id;
+  return id_;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -217,7 +248,7 @@ application::get_id () const
 std::string
 application::get_name () const
 {
-  return data_.name;
+  return name_;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -226,7 +257,7 @@ application::get_name () const
 std::string
 application::get_version () const
 {
-  return data_.version;
+  return version_;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -235,7 +266,7 @@ application::get_version () const
 std::string
 application::get_title () const
 {
-  return data_.title;
+  return title_;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -244,7 +275,7 @@ application::get_title () const
 std::string
 application::get_copyright () const
 {
-  return data_.copyright;
+  return copyright_;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -253,7 +284,7 @@ application::get_copyright () const
 std::string
 application::get_os_name () const
 {
-  return data_.os_name;
+  return os_name_;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -264,7 +295,7 @@ application::get_os_name () const
 std::string
 application::get_data_path (const std::string& path) const
 {
-  return join_path (data_.data_folder, path);
+  return join_path (data_folder_, path);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -275,8 +306,7 @@ application::get_data_path (const std::string& path) const
 std::string
 application::get_config_path (const std::string& path) const
 {
-  load_data ();
-  return join_path (data_.config_folder, path);
+  return join_path (config_folder_, path);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -287,8 +317,7 @@ application::get_config_path (const std::string& path) const
 std::string
 application::get_cache_path (const std::string& path) const
 {
-  load_data ();
-  return join_path (data_.cache_folder, path);
+  return join_path (cache_folder_, path);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -298,7 +327,7 @@ application::get_cache_path (const std::string& path) const
 mobius::database::connection
 application::new_connection ()
 {
-  return data_.pool.acquire ();
+  return connection_pool_.acquire ();
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
