@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -31,7 +31,7 @@
 //! \return new mfc_decoder object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *
-PyMobius_Decoder_Mfc_Decoder_from_cpp (mobius::decoder::mfc_decoder obj)
+pymobius_decoder_mfc_decoder_to_pyobject (mobius::decoder::mfc_decoder obj)
 {
   PyObject *ret = _PyObject_New (&decoder_mfc_decoder_t);
 
@@ -52,8 +52,15 @@ tp_f_skip (decoder_mfc_decoder_o *self, PyObject *args)
   // parse input args
   std::uint64_t arg_size;
 
-  if (!PyArg_ParseTuple (args, "K", &arg_size))
-    return nullptr;
+  try
+    {
+      arg_size = mobius::py::get_arg_as_uint64_t (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -61,9 +68,7 @@ tp_f_skip (decoder_mfc_decoder_o *self, PyObject *args)
   try
     {
       self->obj->skip (arg_size);
-
-      Py_INCREF (Py_None);
-      ret = Py_None;
+      ret = mobius::py::pynone ();
     }
   catch (const std::exception& e)
     {
@@ -113,7 +118,7 @@ tp_f_get_word (decoder_mfc_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_word ());
+      ret = mobius::py::pylong_from_std_uint16_t (self->obj->get_word ());
     }
   catch (const std::exception& e)
     {
@@ -138,7 +143,7 @@ tp_f_get_dword (decoder_mfc_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_dword ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_dword ());
     }
   catch (const std::exception& e)
     {
@@ -163,7 +168,7 @@ tp_f_get_qword (decoder_mfc_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromUnsignedLongLong (self->obj->get_qword ());
+      ret = mobius::py::pylong_from_std_uint64_t (self->obj->get_qword ());
     }
   catch (const std::exception& e)
     {
@@ -188,7 +193,7 @@ tp_f_get_int (decoder_mfc_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromLong (self->obj->get_int ());
+      ret = mobius::py::pylong_from_std_int32_t (self->obj->get_int ());
     }
   catch (const std::exception& e)
     {
@@ -213,7 +218,7 @@ tp_f_get_count (decoder_mfc_decoder_o *self, PyObject *)
 
   try
     {
-      ret = PyLong_FromUnsignedLong (self->obj->get_count ());
+      ret = mobius::py::pylong_from_std_uint32_t (self->obj->get_count ());
     }
   catch (const std::exception& e)
     {
@@ -286,8 +291,15 @@ tp_f_get_hex_string (decoder_mfc_decoder_o *self, PyObject *args)
   // parse input args
   std::uint64_t arg_size;
 
-  if (!PyArg_ParseTuple (args, "K", &arg_size))
-    return nullptr;
+  try
+    {
+      arg_size = mobius::py::get_arg_as_uint64_t (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -301,7 +313,6 @@ tp_f_get_hex_string (decoder_mfc_decoder_o *self, PyObject *args)
       PyErr_SetString (PyExc_Exception, e.what ());
     }
 
-  // return value
   return ret;
 }
 
@@ -342,8 +353,15 @@ tp_f_get_data (decoder_mfc_decoder_o *self, PyObject *args)
   // parse input args
   std::uint64_t arg_size;
 
-  if (!PyArg_ParseTuple (args, "K", &arg_size))
-    return nullptr;
+  try
+    {
+      arg_size = mobius::py::get_arg_as_uint64_t (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -544,19 +562,37 @@ static PyMethodDef tp_methods[] =
 static PyObject *
 tp_new (PyTypeObject *type, PyObject *args, PyObject *)
 {
-  // parse input args
-  PyObject *arg_reader = nullptr;
+  // Parse input args
+  mobius::io::reader arg_reader;
 
-  if (!PyArg_ParseTuple (args, "O", &arg_reader))
-    return nullptr;
+  try
+    {
+      arg_reader = pymobius_io_reader_from_pyobject (mobius::py::get_arg (args, 0));
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
-  // create object
-  decoder_mfc_decoder_o *self = (decoder_mfc_decoder_o *) type->tp_alloc (type, 0);
+  // Create Python object
+  decoder_mfc_decoder_o *ret = reinterpret_cast <decoder_mfc_decoder_o *> (type->tp_alloc (type, 0));
 
-  if (self)
-    self->obj = new mobius::decoder::mfc_decoder (*((io_reader_o *) arg_reader)->obj);
+  if (ret)
+    {
+      try
+        {
+          ret->obj = new mobius::decoder::mfc_decoder (arg_reader);
+        }
+      catch (const std::exception& e)
+        {
+          Py_DECREF (ret);
+          PyErr_SetString (PyExc_Exception, e.what ());
+          ret = nullptr;
+        }
+    }
 
-  return (PyObject *) self;
+  return reinterpret_cast <PyObject *> (ret);
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
