@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -26,14 +26,23 @@
 #include "registry_data.h"
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief registry_key: tp_new (default constructor)
-//! \param type type object
-//! \param args argument list
-//! \param kwds keywords dict
-//! \return new registry_key object or None if object is null
+//! \brief Check if value is an instance of <i>registry_key</i>
+//! \param value Python value
+//! \return true/false
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+bool
+pymobius_os_win_registry_registry_key_check (PyObject *value)
+{
+  return PyObject_IsInstance (value, reinterpret_cast<PyObject *> (&os_win_registry_registry_key_t));
+}
+
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create <b>registry_key</b> Python object from C++ object
+//! \param r Registry key
+//! \return Python object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *
-PyMobius_OS_Win_Registry_Registry_Key_from_cpp (mobius::os::win::registry::registry_key r)
+pymobius_os_win_registry_registry_key_to_pyobject (mobius::os::win::registry::registry_key r)
 {
   PyObject *ret = nullptr;
 
@@ -45,14 +54,25 @@ PyMobius_OS_Win_Registry_Registry_Key_from_cpp (mobius::os::win::registry::regis
         ((os_win_registry_registry_key_o *) ret)->obj = new mobius::os::win::registry::registry_key (r);
     }
   else
-    {
-      Py_INCREF (Py_None);
-      ret = Py_None;
-    }
+    ret = mobius::py::pynone ();
 
   return ret;
 }
 
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+//! \brief Create <b>registry_key</b> C++ object from Python object
+//! \param pyobj Python object
+//! \return C++ object
+// =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+mobius::os::win::registry::registry_key
+pymobius_os_win_registry_registry_key_from_pyobject (PyObject *pyobj)
+{
+  if (!pymobius_os_win_registry_registry_key_check (pyobj))
+    throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("object must be an instance of mobius.os.win.registry_key"));
+
+  return * (reinterpret_cast <os_win_registry_registry_key_o *>(pyobj)->obj);
+}
+
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief registry_key: tp_dealloc
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -127,7 +147,7 @@ tp_getter_classname (os_win_registry_registry_key_o *self)
 
   try
     {
-      ret = PyString2_from_std_string (self->obj->get_classname ());
+      ret = mobius::py::pystring_from_std_string (self->obj->get_classname ());
     }
   catch (const std::exception& e)
     {
@@ -169,7 +189,7 @@ tp_getter_subkeys (os_win_registry_registry_key_o *self)
     {
       ret = mobius::py::pylist_from_cpp_container (
                *(self->obj),
-               PyMobius_OS_Win_Registry_Registry_Key_from_cpp
+               pymobius_os_win_registry_registry_key_to_pyobject
             );
     }
   catch (const std::exception& e)
@@ -192,7 +212,7 @@ tp_getter_values (os_win_registry_registry_key_o *self)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_values (),
-               PyMobius_OS_Win_Registry_Registry_Value_from_cpp
+               pymobius_os_win_registry_registry_value_to_pyobject
             );
     }
   catch (const std::exception& e)
@@ -253,25 +273,32 @@ static PyObject *
 tp_f_add_key (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  os_win_registry_registry_key_o * arg_key;
+  mobius::os::win::registry::registry_key arg_key;
 
-  if (!PyArg_ParseTuple (args, "O", &arg_key))
-    return nullptr;
-
-  // execute C++ function
   try
     {
-      self->obj->add_key (*arg_key->obj);
+      arg_key = mobius::py::get_arg_as_cpp (args, 0, pymobius_os_win_registry_registry_key_from_pyobject);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
+
+  // execute C++ function
+  PyObject *ret = nullptr;
+
+  try
+    {
+      self->obj->add_key (arg_key);
+      ret = mobius::py::pynone ();
     }
   catch (const std::exception& e)
     {
       PyErr_SetString (PyExc_Exception, e.what ());
-      return nullptr;
     }
 
-  // return None
-  Py_INCREF (Py_None);
-  return Py_None;
+  return ret;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -281,25 +308,32 @@ static PyObject *
 tp_f_remove_key (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_name;
+  std::string arg_name;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_name))
-    return nullptr;
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
+  PyObject *ret = nullptr;
+
   try
     {
       self->obj->remove_key (arg_name);
+      ret = mobius::py::pynone ();
     }
   catch (const std::exception& e)
     {
       PyErr_SetString (PyExc_Exception, e.what ());
-      return nullptr;
     }
 
-  // return None
-  Py_INCREF (Py_None);
-  return Py_None;
+  return ret;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -309,19 +343,19 @@ static PyObject *
 tp_f_clear_keys (os_win_registry_registry_key_o *self, PyObject *)
 {
   // execute C++ function
+  PyObject *ret = nullptr;
+
   try
     {
       self->obj->clear_keys ();
+      ret = mobius::py::pynone ();
     }
   catch (const std::exception& e)
     {
       PyErr_SetString (PyExc_Exception, e.what ());
-      return nullptr;
     }
 
-  // return None
-  Py_INCREF (Py_None);
-  return Py_None;
+  return ret;
 }
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -331,17 +365,24 @@ static PyObject *
 tp_f_get_key_by_path (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_path;
+  std::string arg_path;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_path))
-    return nullptr;
+  try
+    {
+      arg_path = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Key_from_cpp (self->obj->get_key_by_path (arg_path));
+      ret = pymobius_os_win_registry_registry_key_to_pyobject (self->obj->get_key_by_path (arg_path));
     }
   catch (const std::exception& e)
     {
@@ -359,17 +400,24 @@ static PyObject *
 tp_f_get_key_by_name (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_path;
+  std::string arg_name;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_path))
-    return nullptr;
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Key_from_cpp (self->obj->get_key_by_name (arg_path));
+      ret = pymobius_os_win_registry_registry_key_to_pyobject (self->obj->get_key_by_name (arg_name));
     }
   catch (const std::exception& e)
     {
@@ -387,10 +435,17 @@ static PyObject *
 tp_f_get_key_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_mask;
+  std::string arg_mask;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_mask))
-    return nullptr;
+  try
+    {
+      arg_mask = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -399,7 +454,7 @@ tp_f_get_key_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_key_by_mask (arg_mask),
-               PyMobius_OS_Win_Registry_Registry_Key_from_cpp
+               pymobius_os_win_registry_registry_key_to_pyobject
             );
     }
   catch (const std::exception& e)
@@ -417,17 +472,24 @@ static PyObject *
 tp_f_get_value_by_path (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_path;
+  std::string arg_path;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_path))
-    return nullptr;
+  try
+    {
+      arg_path = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Value_from_cpp (self->obj->get_value_by_path (arg_path));
+      ret = pymobius_os_win_registry_registry_value_to_pyobject (self->obj->get_value_by_path (arg_path));
     }
   catch (const std::exception& e)
     {
@@ -445,17 +507,24 @@ static PyObject *
 tp_f_get_value_by_name (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_path;
+  std::string arg_name;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_path))
-    return nullptr;
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Value_from_cpp (self->obj->get_value_by_name (arg_path));
+      ret = pymobius_os_win_registry_registry_value_to_pyobject (self->obj->get_value_by_name (arg_name));
     }
   catch (const std::exception& e)
     {
@@ -473,10 +542,17 @@ static PyObject *
 tp_f_get_value_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_mask;
+  std::string arg_mask;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_mask))
-    return nullptr;
+  try
+    {
+      arg_mask = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -485,7 +561,7 @@ tp_f_get_value_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_value_by_mask (arg_mask),
-               PyMobius_OS_Win_Registry_Registry_Value_from_cpp
+               pymobius_os_win_registry_registry_value_to_pyobject
             );
     }
   catch (const std::exception& e)
@@ -503,17 +579,24 @@ static PyObject *
 tp_f_get_data_by_path (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_path;
+  std::string arg_path;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_path))
-    return nullptr;
+  try
+    {
+      arg_path = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Data_from_cpp (self->obj->get_data_by_path (arg_path));
+      ret = pymobius_os_win_registry_registry_data_to_pyobject (self->obj->get_data_by_path (arg_path));
     }
   catch (const std::exception& e)
     {
@@ -531,17 +614,24 @@ static PyObject *
 tp_f_get_data_by_name (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_path;
+  std::string arg_name;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_path))
-    return nullptr;
+  try
+    {
+      arg_name = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
 
   try
     {
-      ret = PyMobius_OS_Win_Registry_Registry_Data_from_cpp (self->obj->get_data_by_name (arg_path));
+      ret = pymobius_os_win_registry_registry_data_to_pyobject (self->obj->get_data_by_name (arg_name));
     }
   catch (const std::exception& e)
     {
@@ -559,10 +649,17 @@ static PyObject *
 tp_f_get_data_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
 {
   // parse input args
-  const char * arg_mask;
+  std::string arg_mask;
 
-  if (!PyArg_ParseTuple (args, "s", &arg_mask))
-    return nullptr;
+  try
+    {
+      arg_mask = mobius::py::get_arg_as_std_string (args, 0);
+    }
+  catch (const std::exception& e)
+    {
+      PyErr_SetString (PyExc_TypeError, e.what ());
+      return nullptr;
+    }
 
   // execute C++ function
   PyObject *ret = nullptr;
@@ -571,7 +668,7 @@ tp_f_get_data_by_mask (os_win_registry_registry_key_o *self, PyObject *args)
     {
       ret = mobius::py::pylist_from_cpp_container (
                self->obj->get_data_by_mask (arg_mask),
-               PyMobius_OS_Win_Registry_Registry_Data_from_cpp
+               pymobius_os_win_registry_registry_data_to_pyobject
             );
     }
   catch (const std::exception& e)
