@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -28,12 +28,12 @@
 #include <stdexcept>
 
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-//! \brief Check if object type is <i>data</i>
+//! \brief Check if object type is an instance of <i>mobius.pod.data</i>
 //! \param pyobj Python object
 //! \return true/false
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 bool
-PyMobius_Pod_Data_Check (PyObject *pyobj)
+pymobius_pod_data_check (PyObject *pyobj)
 {
   return PyObject_IsInstance (pyobj, (PyObject *) &pod_data_t);
 }
@@ -44,15 +44,15 @@ PyMobius_Pod_Data_Check (PyObject *pyobj)
 //! \return new data object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *
-PyMobius_Pod_Data_from_cpp (const mobius::pod::data& obj)
+pymobius_pod_data_to_pyobject (const mobius::pod::data& obj)
 {
   PyObject *ret = nullptr;
   
   if (obj.is_list ())
-    ret = PyMobius_Pod_List_from_cpp (mobius::pod::list (obj));
+    ret = pymobius_pod_list_to_pyobject (mobius::pod::list (obj));
   
   else if (obj.is_map ())
-    ret = PyMobius_Pod_Map_from_cpp (mobius::pod::map (obj));
+    ret = pymobius_pod_map_to_pyobject (mobius::pod::map (obj));
 
   else
     {
@@ -71,7 +71,7 @@ PyMobius_Pod_Data_from_cpp (const mobius::pod::data& obj)
 //! \return data object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::pod::data
-PyMobius_Pod_Data_as_cpp (PyObject *py_value)
+pymobius_pod_data_from_pyobject (PyObject *py_value)
 {
   mobius::pod::data data;
 
@@ -79,7 +79,7 @@ PyMobius_Pod_Data_as_cpp (PyObject *py_value)
     data = * (reinterpret_cast <pod_data_o *>(py_value)->obj);
 
   else
-    data = PyMobius_Pod_Data_from_python (py_value);
+    data = pymobius_pod_data_from_python (py_value);
   
   return data;
 }
@@ -90,21 +90,18 @@ PyMobius_Pod_Data_as_cpp (PyObject *py_value)
 //! \return mobius.pod.data value
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 mobius::pod::data
-PyMobius_Pod_Data_from_python (PyObject *py_value)
+pymobius_pod_data_from_python (PyObject *py_value)
 {
   mobius::pod::data data;
 
-  if (py_value == Py_None)
+  if (mobius::py::pynone_check (py_value))
     ; 
 
   else if (mobius::py::pybool_check (py_value))
     data = mobius::pod::data (py_value == Py_True);
 
-  else if (PyInt_Check (py_value))
-    data = mobius::pod::data (PyInt_AsLong (py_value));
-
-  else if (PyLong_Check (py_value))
-    data = mobius::pod::data (std::int64_t (PyLong_AsLongLong (py_value)));
+  else if (mobius::py::pylong_check (py_value))
+    data = mobius::pod::data (mobius::py::pylong_as_std_int64_t (py_value));
 
   else if (mobius::py::pyfloat_check (py_value))
     data = mobius::pod::data (PyFloat_AS_DOUBLE ( (py_value)));
@@ -119,13 +116,13 @@ PyMobius_Pod_Data_from_python (PyObject *py_value)
     data = mobius::pod::data (mobius::py::pystring_as_std_string (py_value));
 
   else if (PyList_Check (py_value))
-    data = PyMobius_Pod_List_from_python (py_value);
+    data = pymobius_pod_list_from_python (py_value);
 
   else if (PyTuple_Check (py_value))
-    data = PyMobius_Pod_List_from_python (py_value);
+    data = pymobius_pod_list_from_python (py_value);
 
   else if (PyDict_Check (py_value))
-    data = PyMobius_Pod_Map_from_python (py_value);
+    data = pymobius_pod_map_from_python (py_value);
 
   else
     throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("cannot convert value to mobius.pod.data"));
@@ -139,7 +136,7 @@ PyMobius_Pod_Data_from_python (PyObject *py_value)
 //! \return New Python object
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 PyObject *
-PyMobius_Pod_Data_to_python (const mobius::pod::data& value)
+pymobius_pod_data_to_python (const mobius::pod::data& value)
 {
   PyObject *ret = nullptr;
   
@@ -165,10 +162,10 @@ PyMobius_Pod_Data_to_python (const mobius::pod::data& value)
     ret = mobius::py::pybytes_from_bytearray (mobius::bytearray (value));
 
   else if (value.is_list ())
-    ret = PyMobius_Pod_List_to_python (mobius::pod::list (value));
+    ret = pymobius_pod_list_to_python (mobius::pod::list (value));
 
   else if (value.is_map ())
-    ret = PyMobius_Pod_Map_to_python (mobius::pod::map (value));
+    ret = pymobius_pod_map_to_python (mobius::pod::map (value));
 
   else
     throw std::invalid_argument (mobius::MOBIUS_EXCEPTION_MSG ("unknown mobius.pod.data"));
@@ -188,7 +185,7 @@ tp_getter_type (pod_data_o *self)
 
   try
     {
-      ret = PyLong_FromLong (static_cast <int> (self->obj->get_type ()));
+      ret = mobius::py::pylong_from_int (static_cast <int> (self->obj->get_type ()));
     }
   catch (const std::exception& e)
     {
@@ -210,7 +207,7 @@ tp_getter_value (pod_data_o *self)
 
   try
     {
-      ret = PyMobius_Pod_Data_to_python (*(self->obj));
+      ret = pymobius_pod_data_to_python (*(self->obj));
     }
   catch (const std::exception& e)
     {
@@ -256,7 +253,7 @@ tp_f_clone (pod_data_o *self, PyObject *)
 
   try
     {
-      ret = PyMobius_Pod_Data_from_cpp (self->obj->clone ());
+      ret = pymobius_pod_data_to_pyobject (self->obj->clone ());
     }
   catch (const std::exception& e)
     {
@@ -577,9 +574,9 @@ tp_new (PyTypeObject *, PyObject *args, PyObject *)
       mobius::pod::data data;
 
       if (mobius::py::get_arg_size (args) > 0)
-        data = PyMobius_Pod_Data_from_python (mobius::py::get_arg (args, 0));
+        data = pymobius_pod_data_from_python (mobius::py::get_arg (args, 0));
 
-      ret = PyMobius_Pod_Data_from_cpp (data);
+      ret = pymobius_pod_data_to_pyobject (data);
     }
   catch (const std::exception& e)
     {
